### Name: hooks
### Title: User-defined Hooks.
### Aliases: hooks callbacks on.select.element on.select.annotation
###   on.select.region on.select.points figure.startup simple.calculus
###   spline.curve spline.points move.indexed move.indexed.y
###   delete.in.range xsort.curve smooth.curve lf.in.range default.hook
###   default.annotation default.hooks
### Keywords: hplot iplot dynamic

### ** Examples

#example of hook suitable for multi-gauss peak fitting.

#fit Gauss peaks with linear bias 
bias.gauss<-function(par,x,y){
  y1<-NULL
    for (i in 1:dim(par)[1]) {y1<-cbind(y1,exp(-(x-par[i,1])^2/par[i,2]^2))}
    Y<-cbind(y1,x,1)
    lf<-lm.fit(Y,y)
    return(list(lf=lf,Y=Y))
}

#wrapper for bias.gauss
fitfun<-function(par,x,y,dimensions){
  dim(par)<-dimensions
  i<-bias.gauss(par,x,y);
  sum(i$lf$residuals^2)
}

#the hook itself
#it is expected that .DATA global variable already holds 
#the data we want to fit. This means either on.select.element or
#on.select.region is set to default.hook(). Then user should select points 
#where he would expect the peaks maxima to be located. 
#When done, hitting "Enter" key will start the fitting itself.
on.select.points <- function(reg,handle,index,data){
  #Test all peaks have been selected.
  if (is.null(reg)){
    #first guess - not very smart
    par <- cbind(data[,1],data[,1]/100.)
    #optimize
    Parm<<-optim(par,fitfun,x=.DATA[,1],y=.DATA[,2],dimensions=dim(par));
    dim(Parm$par)<-dim(par)
    #calculate y-values of single peak components
    l1<-bias.gauss(Parm$par,x=.DATA[,1],y=.DATA[,2])
    #plot fitted curve - the peaks sum
    ggplot(x=.DATA[,1],y=l1$Y%*%l1$lf$coefficients,gp=gpar(col="red"))
    }    
}
  



